--[[
    BGLive is an user interface for spectated battleground 
	wargames in World of Warcraft.
    Copyright (C) 2015  Harald Böhm <harald@boehm.agency>
	Further contributors: Jochen Taeschner and Romina Schmidt.
	
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	ADDITIONAL PERMISSION UNDER GNU GPL VERSION 3 SECTION 7:
	As a special exception, the copyright holder of this add-on gives you
	permission to link this add-on with independent proprietary software,
	regardless of the license terms of the independent proprietary software.
]]

-- ArenaLive addon Name and localisation table:
local addonName, L = ...;

--[[
**************************************************
******* GENERAL HANDLER SET UP STARTS HERE *******
**************************************************
]]--
local ObjectiveIcon = ArenaLive:ConstructHandler("ObjectiveIcon", true, false);
ObjectiveIcon:RegisterEvent("UNIT_AURA");

local OBJECTIVE_SPELLIDS = {
	[121164] = "DEBUFF",	-- Kotmogu Orb (Blue)
	[156621] = "BUFF",		-- Alliance Flag
	[156618] = "BUFF",		-- Horde Flag
	[34976] = "BUFF",		-- Netherstorm Flag
	[140876] = "BUFF",		-- Alliance Mine Cart
	[141210] = "BUFF",		-- Horde Mine Cart
};

local OBJECTIVE_STACKS_SPELLIDS = {
	[156621] = { -- Alliance Flag
		[46392] = "Focused Assault",
		[46393] = "Brutal Assault",	
	};		
	[156618] = { -- Horde Flag
		[46392] = "Focused Assault",
		[46393] = "Brutal Assault",
	},
	[34976] = { -- Netherstorm Flag
		[46392] = "Focused Assault",
		[46393] = "Brutal Assault",
	},
	[140876] = { -- Alliance Mine Cart
		[46392] = "Focused Assault",
		[46393] = "Brutal Assault",
	},
	[141210] = { -- Horde Mine Cart
		[46392] = "Focused Assault",
		[46393] = "Brutal Assault",
	},
};

local OBJECTIVE_STACK_MULTIPLIER = {
	[156621] = 10,		-- Alliance Flag
	[156618] = 10,		-- Horde Flag
	[34976] = 10,		-- Netherstorm Flag
	[140876] = 10,		-- Alliance Mine Cart
	[141210] = 10,		-- Horde Mine Cart
	[121164] = 1;		-- Blue Orb
	[121175] = 1;		-- Purple Orb
	[121176] = 1;		-- Green Orb 
	[121177] = 1;		-- Orange Orb
};

local OBJECTIVE_ALT_TEXTURES = {
	[121164] = "Interface\\AddOns\\BGLive\\Textures\\BlueOrb";		-- Blue Orb
	[121175] = "Interface\\AddOns\\BGLive\\Textures\\PurpleOrb";	-- Purple Orb
	[121176] = "Interface\\AddOns\\BGLive\\Textures\\GreenOrb";		-- Green Orb 
	[121177] = "Interface\\Icons\\INV_Misc_Orb_05";					-- Orange Orb
};

local GUID_TO_OBJECTIVE = {};
local GUID_TO_OBJECTIVE_STACKS = {};
--[[
****************************************
****** HANDLER METHODS START HERE ******
****************************************
]]--
function ObjectiveIcon:ConstructObject(icon, texture, text)
	icon.texture = texture;
	icon.text = text;
end

function ObjectiveIcon:Update(unitFrame)
	
	local unit = unitFrame.unit;
	local icon = unitFrame[self.name];
	if ( not unit or not icon ) then
		return;
	end
	
	local guid = UnitGUID(unit);
	if ( guid and GUID_TO_OBJECTIVE[guid] ) then
		local spellID = GUID_TO_OBJECTIVE[guid];
		local name, rank, texture = GetSpellInfo(spellID);
		icon.texture:SetTexture(OBJECTIVE_ALT_TEXTURES[spellID] or texture);
		
		-- Get debuff stacks:
		if ( GUID_TO_OBJECTIVE_STACKS[guid] ) then
			local stacks = GUID_TO_OBJECTIVE_STACKS[guid];
			if ( stacks > 1 ) then
				
				local red, green, blue;
				if ( stacks < 25 ) then
					red, green, blue = 0, 1, 0;
				elseif ( stacks < 50 ) then
					red, green, blue = 1, 1, 0;
				elseif ( stacks < 75 ) then
					red, green, blue = 1, 0.44, 0;
				else
					red, green, blue = 1, 0, 0;
				end
				
				icon.text:SetText(stacks.."%");
				icon.text:SetTextColor(red, green, blue);
				icon.text:Show();
			else
				icon.text:Hide();
			end
		end
		
		icon:Show();
	else
		self:Reset(unitFrame);
	end

end

function ObjectiveIcon:Reset(unitFrame)
	local icon = unitFrame[self.name];
	icon:Hide();
end

function ObjectiveIcon:UnitObjectiveInfo(unit, spellID)
	
	if ( OBJECTIVE_SPELLIDS[spellID] ) then
		local spellName = GetSpellInfo(spellID);
		if ( not spellName ) then
			return nil, nil, nil;
		end
		
		local auraType = OBJECTIVE_SPELLIDS[spellID];
		
		local name, _, _, count, value;
		if ( auraType == "BUFF" ) then
			name, _, _, count, _, _, _, _, _, _, newSpellID, _, _, _, value1, value2, value3 = UnitBuff(unit, spellName);
		elseif ( auraType == "DEBUFF" ) then
			name, _, _, count, _, _, _, _, _, _, newSpellID, _, _, _, value1, value2, value3 = UnitDebuff(unit, spellName);
		end
		
		if ( name ) then
			if ( OBJECTIVE_STACKS_SPELLIDS[spellID] ) then
				for stackSpellID in pairs(OBJECTIVE_STACKS_SPELLIDS[spellID]) do
					local stackName = GetSpellInfo(stackSpellID);
					local _, _, _, stackCount, _, _, _, _, _, _, stackSpellID = UnitDebuff(unit, stackName);
					if ( stackCount ) then
						count = stackCount;
						break;
					end
				end
			elseif ( spellID == 121164 or spellID == 121175 or spellID == 121176 or spellID == 121177 ) then
				-- Kotmogu orbs, use value for increased damage taken:
				count = value2;
			end
			
			count =  count * OBJECTIVE_STACK_MULTIPLIER[spellID];
			return newSpellID, name, count;
		else
			return nil, nil, nil;
		end
		
		
	else
		return nil, nil, nil;
	end

end

function ObjectiveIcon:OnEvent(event, ...)
	if ( event == "UNIT_AURA" ) then
		local unit = ...;
		local guid = UnitGUID(unit);
		if ( guid ) then
			local objective;
			for spellID in pairs(OBJECTIVE_SPELLIDS) do
				local newSpellID, name, stacks = self:UnitObjectiveInfo(unit, spellID)
				if ( name ) then
					GUID_TO_OBJECTIVE[guid] = newSpellID;
					GUID_TO_OBJECTIVE_STACKS[guid] = stacks;
					objective = true;
					break;
				end
			end
			
			if ( not objective ) then
				GUID_TO_OBJECTIVE[guid] = nil;
				GUID_TO_OBJECTIVE_STACKS[guid] = nil;
			end
		end
		
		if ( ArenaLive:IsUnitInUnitFrameCache(unit) ) then
			for id in ArenaLive:GetAffectedUnitFramesByUnit(unit) do
				local unitFrame = ArenaLive:GetUnitFrameByID(id);
				if ( unitFrame[self.name] ) then
					self:Update(unitFrame);
				end
			end
		end
	end
end